/*
 * Copyright (c) 2021 DuckDuckGo
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

@file:Suppress("MemberVisibilityCanBePrivate")

package com.duckduckgo.mobile.android.ui.view

import android.content.Context
import android.util.AttributeSet
import android.widget.LinearLayout
import com.duckduckgo.mobile.android.R
import com.duckduckgo.mobile.android.databinding.ViewSettingsOptionItemBinding
import com.duckduckgo.mobile.android.ui.viewbinding.viewBinding

class SettingsOption : LinearLayout {

    private val binding: ViewSettingsOptionItemBinding by viewBinding()

    constructor(context: Context) : this(context, null)
    constructor(
        context: Context,
        attrs: AttributeSet?
    ) : this(context, attrs, R.style.Widget_DuckDuckGo_SettingsOption)

    constructor(
        context: Context,
        attrs: AttributeSet?,
        defStyle: Int
    ) : super(context, attrs, defStyle) {
        val attributes = context.obtainStyledAttributes(attrs, R.styleable.SettingsOption)
        binding.root.setBackgroundResource(
            attributes.getResourceId(
                R.styleable.SearchBarView_android_background,
                context.defaultSelectableItemBackground()
            )
        )

        binding.title.setTextColor(
            attributes.getColor(
                R.styleable.SettingsOption_settingsOptionColor,
                context.getColorFromAttr(R.attr.normalTextColor)
            )
        )

        setTitle(attributes.getString(R.styleable.SettingsOption_android_text) ?: "")
        attributes.recycle()
    }

    /** Sets the item title */
    fun setTitle(title: String) {
        binding.title.text = title
    }

    /** Sets the item click listener */
    fun setClickListener(onClick: () -> Unit) {
        binding.itemContainer.setOnClickListener { onClick() }
    }
}
